@@ -15,11 +15,12 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 
 import itertools
 from nml import generic, nmlop
-from nml.expression import BinOp, ConstantNumeric, ConstantFloat, Array, StringLiteral, Identifier
+from nml.expression import (BinOp, ConstantNumeric, ConstantFloat, Array, StringLiteral,
+                            Identifier, ProduceCargo, AcceptCargo, parse_string_to_dword)
 
 tilelayout_names = {}
 
-class BaseAction0Property(object):
+class BaseAction0Property:
     """
     Base class for Action0 properties.
     """
@@ -151,7 +152,7 @@ class Action0Property(BaseAction0Property):
 #
 # 'first' (value doesn't matter) if the property should be set first (generally a substitute type)
 
-properties = 0x12 * [None]
+properties = 0x14 * [None]
 
 #
 # Some helper functions that are used for multiple features
@@ -255,14 +256,18 @@ general_veh_props = {
 }
 
 def ottd_display_speed(value, divisor, unit):
-    return int(value.value / divisor) * 10 // 16 * unit.ottd_mul >> unit.ottd_shift
+    return (round(value.value / divisor * 10 * unit.ottd_mul) >> unit.ottd_shift) // 16
 
-class CargotypeListProp(BaseAction0Property):
-    def __init__(self, prop_num, data):
+class VariableListProp(BaseAction0Property):
+    """
+    Property value that is a variable-length list of variable sized values, the list length is written before the data.
+    """
+    def __init__(self, prop_num, data, size):
         # data is a list, each element belongs to an item ID
         # Each element in the list is a list of cargo types
         self.prop_num = prop_num
         self.data = data
+        self.size = size
 
     def write(self, file):
         file.print_bytex(self.prop_num)
@@ -270,23 +275,46 @@ class CargotypeListProp(BaseAction0Property):
             file.print_byte(len(elem))
             for i, val in enumerate(elem):
                 if i % 8 == 0: file.newline()
-                file.print_bytex(val)
+                file.print_varx(val, self.size)
             file.newline()
 
     def get_size(self):
         total_len = 1 # Prop number
         for elem in self.data:
             # For each item ID to set, make space for all values + 1 for the length
-            total_len += len(elem) + 1
+            total_len += len(elem) * self.size + 1
         return total_len
 
+def VariableByteListProp(prop_num, data):
+    return VariableListProp(prop_num, data, 1)
+
 def ctt_list(prop_num, *values):
     # values may have multiple entries, if more than one item ID is set (e.g. multitile houses)
     # Each value is an expression.Array of cargo types
     for value in values:
         if not isinstance(value, Array):
             raise generic.ScriptError("Value of cargolist property must be an array", value.pos)
-    return [CargotypeListProp(prop_num, [[ctype.reduce_constant().value for ctype in single_item_array.values] for single_item_array in values])]
+    return [VariableByteListProp(prop_num, [[ctype.reduce_constant().value for ctype in single_item_array.values] for single_item_array in values])]
+
+def VariableWordListProp(num_prop, data):
+    return VariableListProp(num_prop, data, 2)
+
+def accepted_cargos(prop_num, *values):
+    # values may have multiple entries, if more than one item ID is set (e.g. multitile houses)
+    # Each value is an expression.Array of cargo types and amount arrays
+    cargos = []
+    for value in values:
+        if not isinstance(value, Array) or len(value.values) > 16:
+            raise generic.ScriptError("accepted_cargos must be an array with no more than 16 values", value.pos)
+        tile_cargos = []
+        for cargo_amount_pair in value.values:
+            if not isinstance(cargo_amount_pair, Array) or len(cargo_amount_pair.values) != 2:
+                raise generic.ScriptError("Each element of accepted_cargos must be an array with two elements: cargoid and amount", cargo_amount_pair.pos)
+            cargo_id = cargo_amount_pair.values[0].reduce_constant().value
+            cargo_amount = cargo_amount_pair.values[1].reduce_constant().value
+            tile_cargos.append((cargo_amount << 8) | cargo_id)
+        cargos.append(tile_cargos)
+    return [VariableWordListProp(prop_num, cargos)]
 
 def vehicle_length(value):
     if isinstance(value, ConstantNumeric):
@@ -361,6 +389,8 @@ def roadveh_speed_prop(prop_info):
     return [prop08, prop15]
 
 properties[0x01] = {
+    'road_type'                    : {'size': 1, 'num': 0x05},
+    'tram_type'                    : {'size': 1, 'num': 0x05},
     'speed'                        : roadveh_speed_prop({'unit_type': 'speed', 'unit_conversion': (10000, 1397), 'adjust_value': lambda val, unit: ottd_display_speed(val, 2, unit)}),
     'running_cost_factor'          : {'size': 1, 'num': 0x09},
     'running_cost_base'            : {'size': 4, 'num': 0x0A},
@@ -515,42 +545,6 @@ def house_prop_21_22(value, index):
         raise generic.ScriptError("Availability years must be an array with exactly two values", value.pos)
     return value.values[index]
 
-def house_acceptance(value, index):
-    if not isinstance(value, Array) or len(value.values) > 3:
-        raise generic.ScriptError("accepted_cargos must be an array with no more than 3 values", value.pos)
-
-    if index < len(value.values):
-        cargo_amount_pair = value.values[index]
-        if not isinstance(cargo_amount_pair, Array) or len(cargo_amount_pair.values) != 2:
-            raise generic.ScriptError("Each element of accepted_cargos must be an array with two elements: cargoid and amount", cargo_amount_pair.pos)
-        return cargo_amount_pair.values[1]
-    else:
-        return ConstantNumeric(0, value.pos)
-
-def house_accepted_cargo_types(value):
-    if not isinstance(value, Array) or len(value.values) > 3:
-        raise generic.ScriptError("accepted_cargos must be an array with no more than 3 values", value.pos)
-
-    cargoes = []
-    for i in range(3):
-        if i < len(value.values):
-            cargo_amount_pair = value.values[i]
-            if not isinstance(cargo_amount_pair, Array) or len(cargo_amount_pair.values) != 2:
-                raise generic.ScriptError("Each element of accepted_cargos must be an array with two elements: cargoid and amount", cargo_amount_pair.pos)
-            cargoes.append(cargo_amount_pair.values[0])
-        else:
-            cargoes.append(ConstantNumeric(0xFF, value.pos))
-
-    ret = None
-    for i, cargo in enumerate(cargoes):
-        byte = BinOp(nmlop.AND, cargo, ConstantNumeric(0xFF, cargo.pos), cargo.pos)
-        if i == 0:
-            ret = byte
-        else:
-            byte = BinOp(nmlop.SHIFT_LEFT, byte, ConstantNumeric(i * 8, cargo.pos), cargo.pos)
-            ret = BinOp(nmlop.OR, ret, byte, cargo.pos)
-    return ret.reduce()
-
 def house_random_colours(value):
     # User sets array with 4 values (range 0..15)
     # Output is a dword, each byte being a value from the array
@@ -640,10 +634,7 @@ properties[0x07] = {
                                  {'size': 2, 'num': 0x22, 'multitile_function': mt_house_zero, 'value_function': lambda value: house_prop_21_22(value, 1)}],
     'population'              : {'size': 1, 'num': 0x0B, 'multitile_function': mt_house_zero},
     'mail_multiplier'         : {'size': 1, 'num': 0x0C, 'multitile_function': mt_house_zero},
-    'accepted_cargos'         : [{'size': 1, 'num': 0x0D, 'multitile_function': mt_house_same, 'value_function': lambda value: house_acceptance(value, 0)},
-                                 {'size': 1, 'num': 0x0E, 'multitile_function': mt_house_same, 'value_function': lambda value: house_acceptance(value, 1)},
-                                 {'size': 1, 'num': 0x0F, 'multitile_function': mt_house_same, 'value_function': lambda value: house_acceptance(value, 2)},
-                                 {'size': 4, 'num': 0x1E, 'multitile_function': mt_house_same, 'value_function': house_accepted_cargo_types}],
+    # prop 0D - 0F are replaced by prop 23
     'local_authority_impact'  : {'size': 2, 'num': 0x10, 'multitile_function': mt_house_same},
     'removal_cost_multiplier' : {'size': 1, 'num': 0x11, 'multitile_function': mt_house_same},
     'name'                    : {'size': 2, 'num': 0x12, 'string': 0xDC, 'multitile_function': mt_house_same},
@@ -658,9 +649,11 @@ properties[0x07] = {
     'animation_speed'         : {'size': 1, 'num': 0x1B, 'multitile_function': mt_house_same},
     'building_class'          : {'size': 1, 'num': 0x1C, 'multitile_function': mt_house_class},
     # prop 1D (callback flags 2) is not set by user
+    # prop 1E is replaced by prop 23
     'minimum_lifetime'        : {'size': 1, 'num': 0x1F, 'multitile_function': mt_house_zero},
-    'watched_cargo_types'     : {'multitile_function': mt_house_same, 'custom_function': lambda *values: ctt_list(0x20, *values)}
+    'watched_cargo_types'     : {'multitile_function': mt_house_same, 'custom_function': lambda *values: ctt_list(0x20, *values)},
     # prop 21 -22 see above (years_available, prop 0A)
+    'accepted_cargos'         : {'multitile_function': mt_house_same, 'custom_function': lambda *values: accepted_cargos(0x23, *values)},
 }
 
 # Feature 0x08 (General Vars) is implemented elsewhere (e.g. basecost, snowline)
@@ -669,34 +662,17 @@ properties[0x07] = {
 # Feature 0x09 (Industry Tiles)
 #
 
-def industrytile_cargos(value):
-    if not isinstance(value, Array) or len(value.values) > 3:
-        raise generic.ScriptError("accepted_cargos must be an array with no more than 3 values", value.pos)
-    prop_num = 0x0A
-    props = []
-    for cargo_amount_pair in value.values:
-        if not isinstance(cargo_amount_pair, Array) or len(cargo_amount_pair.values) != 2:
-            raise generic.ScriptError("Each element of accepted_cargos must be an array with two elements: cargoid and amount", cargo_amount_pair.pos)
-        cargo_id = cargo_amount_pair.values[0].reduce_constant().value
-        cargo_amount = cargo_amount_pair.values[1].reduce_constant().value
-        props.append(Action0Property(prop_num, ConstantNumeric((cargo_amount << 8) | cargo_id), 2))
-        prop_num += 1
-
-    while prop_num <= 0x0C:
-        props.append(Action0Property(prop_num, ConstantNumeric(0), 2))
-        prop_num += 1
-    return props
-
 properties[0x09] = {
     'substitute'         : {'size': 1, 'num': 0x08, 'first': None},
     'override'           : {'size': 1, 'num': 0x09},
-    'accepted_cargos'    : {'custom_function': industrytile_cargos}, # = prop 0A - 0C
+    # prop 0A - 0C are replaced by prop 13
     'land_shape_flags'   : {'size': 1, 'num': 0x0D},
     # prop 0E (callback flags) is not set by user
     'animation_info'     : {'size': 2, 'num': 0x0F, 'value_function': animation_info},
     'animation_speed'    : {'size': 1, 'num': 0x10},
     'animation_triggers' : {'size': 1, 'num': 0x11},
     'special_flags'      : {'size': 1, 'num': 0x12},
+    'accepted_cargos'    : {'custom_function': lambda value: accepted_cargos(0x13, value)},
 }
 
 #
@@ -733,15 +709,6 @@ def industry_layouts(value):
         layouts.append(tilelayout_names[name.value])
     return [IndustryLayoutProp(layouts)]
 
-def industry_prod_multiplier(value):
-    if not isinstance(value, Array) or len(value.values) > 2:
-        raise generic.ScriptError("Prod multiplier must be an array of up to two values", value.pos)
-    props = []
-    for i in range(0, 2):
-        val = value.values[i].reduce_constant() if i < len(value.values) else ConstantNumeric(0)
-        props.append(Action0Property(0x12 + i, val, 1))
-    return props
-
 class RandomSoundsProp(BaseAction0Property):
     def __init__(self, sound_list):
         self.sound_list = sound_list
@@ -788,18 +755,90 @@ def industry_conflicting_types(value):
         types_list.append(ConstantNumeric(0xFF))
     return [ConflictingTypesProp(types_list)]
 
-def industry_input_multiplier(value, prop_num):
-    if not isinstance(value, Array) or len(value.values) > 2:
-        raise generic.ScriptError("Input multiplier must be an array of up to two values", value.pos)
-    val1 = value.values[0].reduce() if len(value.values) > 0 else ConstantNumeric(0)
-    val2 = value.values[1].reduce() if len(value.values) > 1 else ConstantNumeric(0)
-    if not isinstance(val1, (ConstantNumeric, ConstantFloat)) or not isinstance(val2, (ConstantNumeric, ConstantFloat)):
-        raise generic.ScriptError("Expected a compile-time constant", value.pos)
-    generic.check_range(val1.value, 0, 256, "input_multiplier", val1.pos)
-    generic.check_range(val2.value, 0, 256, "input_multiplier", val2.pos)
-    mul1 = int(val1.value * 256)
-    mul2 = int(val2.value * 256)
-    return [Action0Property(prop_num, ConstantNumeric(mul1 | (mul2 << 16)), 4)]
+class IndustryInputMultiplierProp(BaseAction0Property):
+    def __init__(self, prop_num, data):
+        self.prop_num = prop_num
+        self.data = data
+
+    def write(self, file):
+        file.print_bytex(self.prop_num)
+        if len(self.data) == 0:
+            file.print_byte(0)
+            file.print_byte(0)
+        else:
+            file.print_byte(len(self.data))
+            file.print_byte(len(self.data[0])) # assume all sub-arrays are equal length
+            file.newline()
+            for out_muls in self.data:
+                for mul in out_muls:
+                    file.print_wordx(mul)
+                file.newline()
+
+    def get_size(self):
+        if len(self.data) == 0:
+            return 3
+        else:
+            return 3 + len(self.data) * len(self.data[0]) * 2
+
+def industry_cargo_types(value):
+    if isinstance(value, Array):
+        cargo_types = value.values
+    else:
+        cargo_types = [value]
+    if not all(isinstance(item, (ProduceCargo, AcceptCargo)) for item in value.values):
+        raise generic.ScriptError("Cargo types definition must be an array produce_cargo() and accept_cargo() expressions", value.pos)
+
+    # collect all the cargo types involved
+    input_cargos = []
+    output_cargos = []
+    def check_produce(prd):
+        if len(prd.value) != 1:
+            raise generic.ScriptError("Cargo types produce_cargo() expressions require 2 arguments", prd.pos)
+        if not isinstance(prd.value[0], (ConstantNumeric, ConstantFloat)):
+            raise generic.ScriptError("Cargo types produce_cargo() expressions must have numeric constant values", prd.pos)
+        if prd.cargotype not in output_cargos: output_cargos.append(prd.cargotype)
+    def check_accept(acp):
+        if item.cargotype not in input_cargos: input_cargos.append(item.cargotype)
+        for outitem in item.value:
+            if isinstance(outitem, ProduceCargo):
+                check_produce(outitem)
+            else:
+                raise generic.ScriptError("Cargo types accept_cargo() expressions must only contain produce_cargo() expressions", outitem.pos)
+    for item in cargo_types:
+        # use "if not in: append" idiom rather than sets to preserve ordering of cargotypes between NML and NFO
+        if isinstance(item, ProduceCargo):
+            check_produce(item)
+        elif isinstance(item, AcceptCargo):
+            check_accept(item)
+        else:
+            assert False
+
+    if len(input_cargos) > 16:
+        raise generic.ScriptError("Cargo types definition contains more than 16 different accept_cargo() cargotypes", value.pos)
+    if len(output_cargos) > 16:
+        raise generic.ScriptError("Cargo types definition contains more than 16 different produce_cargo() cargotypes", value.pos)
+
+    # prepare lists for the remaining output properties
+    prod_multipliers = [0 for cargo in output_cargos]
+    input_multipliers = [ [0 for outcargo in output_cargos] for incargo in input_cargos ]
+    has_inpmult = False
+
+    # populate prod_multipliers and input_multipliers
+    for item in cargo_types:
+        if isinstance(item, ProduceCargo):
+            prod_multipliers[output_cargos.index(item.cargotype)] = int(item.value[0].value)
+        elif isinstance(item, AcceptCargo):
+            row = input_multipliers[input_cargos.index(item.cargotype)]
+            for outitem in item.value:
+                row[output_cargos.index(outitem.cargotype)] = int(outitem.value[0].value * 256)
+                if outitem.value[0].value > 0: has_inpmult = True
+
+    return [
+        VariableByteListProp(0x25, [output_cargos]),
+        VariableByteListProp(0x26, [input_cargos]),
+        VariableByteListProp(0x27, [prod_multipliers]),
+        IndustryInputMultiplierProp(0x28, input_multipliers if has_inpmult else [])
+    ]
 
 properties[0x0A] = {
     'substitute'             : {'size': 1, 'num': 0x08, 'first': None},
@@ -810,9 +849,7 @@ properties[0x0A] = {
     'prod_increase_msg'      : {'size': 2, 'num': 0x0D, 'string': 0xDC},
     'prod_decrease_msg'      : {'size': 2, 'num': 0x0E, 'string': 0xDC},
     'fund_cost_multiplier'   : {'size': 1, 'num': 0x0F},
-    'prod_cargo_types'       : {'size': 2, 'num': 0x10, 'value_function': lambda value: cargo_list(value, 2)},
-    'accept_cargo_types'     : {'size': 4, 'num': 0x11, 'value_function': lambda value: cargo_list(value, 3)},
-    'prod_multiplier'        : {'custom_function': industry_prod_multiplier}, # = prop 12,13
+    'cargo_types'            : {'custom_function': industry_cargo_types}, # = prop 25+26+27+28 combined in one structure
     'min_cargo_distr'        : {'size': 1, 'num': 0x14},
     'random_sound_effects'   : {'custom_function': random_sounds}, # = prop 15
     'conflicting_ind_types'  : {'custom_function': industry_conflicting_types}, # = prop 16
@@ -822,9 +859,6 @@ properties[0x0A] = {
     'map_colour'             : {'size': 1, 'num': 0x19},
     'spec_flags'             : {'size': 4, 'num': 0x1A},
     'new_ind_msg'            : {'size': 2, 'num': 0x1B, 'string': 0xDC},
-    'input_multiplier_1'     : {'custom_function': lambda value: industry_input_multiplier(value, 0x1C)},
-    'input_multiplier_2'     : {'custom_function': lambda value: industry_input_multiplier(value, 0x1D)},
-    'input_multiplier_3'     : {'custom_function': lambda value: industry_input_multiplier(value, 0x1E)},
     'name'                   : {'size': 2, 'num': 0x1F, 'string': 0xDC},
     'prospect_chance'        : {'size': 4, 'num': 0x20, 'unit_conversion': 0xFFFFFFFF},
     # prop 21, 22 (callback flags) are not set by user
@@ -959,54 +993,62 @@ properties[0x0F] = {
 }
 
 #
-# Feature 0x10 (Rail Types)
+# General tracktype properties that apply to features 0x10 & 0x12/13 (rail/road/tramtypes)
 #
 
-class RailtypeListProp(BaseAction0Property):
-    def __init__(self, prop_num, railtype_list):
+class LabelListProp(BaseAction0Property):
+    def __init__(self, prop_num, labels):
         self.prop_num = prop_num
-        self.railtype_list = railtype_list
+        self.labels = labels
 
     def write(self, file):
         file.print_bytex(self.prop_num)
-        file.print_byte(len(self.railtype_list))
-        for railtype in self.railtype_list:
-            railtype.write(file, 4)
+        file.print_byte(len(self.labels))
+        for label in self.labels:
+            parse_string_to_dword(label)  # Error if the wrong length or not ASCII
+            label.write(file, 4)
         file.newline()
 
     def get_size(self):
-        return len(self.railtype_list) * 4 + 2
+        return len(self.labels) * 4 + 2
 
-def railtype_list(value, prop_num):
+
+def label_list(value, prop_num, description):
     if not isinstance(value, Array):
-        raise generic.ScriptError("Railtype list must be an array of literal strings", value.pos)
-    for val in value.values:
-        if not isinstance(val, StringLiteral): raise generic.ScriptError("Railtype list must be an array of literal strings", val.pos)
-    return [RailtypeListProp(prop_num, value.values)]
+        raise generic.ScriptError(description + " list must be an array of literal strings", value.pos)
+    return [LabelListProp(prop_num, value.values)]
 
-properties[0x10] = {
+common_tracktype_props = {
     'label'                    : {'size': 4, 'num': 0x08, 'string_literal': 4}, # is allocated during reservation stage, setting label first is thus not needed
     'toolbar_caption'          : {'size': 2, 'num': 0x09, 'string': 0xDC},
     'menu_text'                : {'size': 2, 'num': 0x0A, 'string': 0xDC},
     'build_window_caption'     : {'size': 2, 'num': 0x0B, 'string': 0xDC},
     'autoreplace_text'         : {'size': 2, 'num': 0x0C, 'string': 0xDC},
     'new_engine_text'          : {'size': 2, 'num': 0x0D, 'string': 0xDC},
-    'compatible_railtype_list' : {'custom_function': lambda x: railtype_list(x, 0x0E)},
-    'powered_railtype_list'    : {'custom_function': lambda x: railtype_list(x, 0x0F)},
-    'railtype_flags'           : {'size': 1, 'num': 0x10},
-    'curve_speed_multiplier'   : {'size': 1, 'num': 0x11},
-    'station_graphics'         : {'size': 1, 'num': 0x12},
     'construction_cost'        : {'size': 2, 'num': 0x13},
-    'speed_limit'              : {'size': 2, 'num': 0x14, 'unit_type': 'speed', 'unit_conversion': (5000, 1397)},
-    'acceleration_model'       : {'size': 1, 'num': 0x15},
     'map_colour'               : {'size': 1, 'num': 0x16},
     'introduction_date'        : {'size': 4, 'num': 0x17},
-    'requires_railtype_list'   : {'custom_function': lambda x: railtype_list(x, 0x18)},
-    'introduces_railtype_list' : {'custom_function': lambda x: railtype_list(x, 0x19)},
     'sort_order'               : {'size': 1, 'num': 0x1A},
     'name'                     : {'size': 2, 'num': 0x1B, 'string': 0xDC},
     'maintenance_cost'         : {'size': 2, 'num': 0x1C},
-    'alternative_railtype_list': {'custom_function': lambda x: railtype_list(x, 0x1D)},
+}
+
+#
+# Feature 0x10 (Rail Types)
+#
+
+properties[0x10] = {
+    **common_tracktype_props,
+    'compatible_railtype_list' : {'custom_function': lambda x: label_list(x, 0x0E, "Railtype")},
+    'powered_railtype_list'    : {'custom_function': lambda x: label_list(x, 0x0F, "Railtype")},
+    'railtype_flags'           : {'size': 1, 'num': 0x10},
+    'curve_speed_multiplier'   : {'size': 1, 'num': 0x11},
+    'station_graphics'         : {'size': 1, 'num': 0x12},
+    'speed_limit'              : {'size': 2, 'num': 0x14, 'unit_type': 'speed', 'unit_conversion': (5000, 1397)},
+    'acceleration_model'       : {'size': 1, 'num': 0x15},
+    'requires_railtype_list'   : {'custom_function': lambda x: label_list(x, 0x18, "Railtype")},
+    'introduces_railtype_list' : {'custom_function': lambda x: label_list(x, 0x19, "Railtype")},
+    'alternative_railtype_list': {'custom_function': lambda x: label_list(x, 0x1D, "Railtype")},
 }
 
 #
@@ -1022,3 +1064,31 @@ properties[0x11] = {
     'animation_speed'    : {'size': 1, 'num': 0x10},
     'animation_triggers' : {'size': 1, 'num': 0x11},
 }
+
+#
+# Feature 0x12 (Road Types)
+#
+
+properties[0x12] = {
+    **common_tracktype_props,
+    'powered_roadtype_list'    : {'custom_function': lambda x: label_list(x, 0x0F, "Roadtype")},
+    'roadtype_flags'           : {'size': 1, 'num': 0x10},
+    'speed_limit'              : {'size': 2, 'num': 0x14, 'unit_type': 'speed', 'unit_conversion': (10000, 1397)},
+    'requires_roadtype_list'   : {'custom_function': lambda x: label_list(x, 0x18, "Roadtype")},
+    'introduces_roadtype_list' : {'custom_function': lambda x: label_list(x, 0x19, "Roadtype")},
+    'alternative_roadtype_list': {'custom_function': lambda x: label_list(x, 0x1D, "Roadtype")},
+}
+
+#
+# Feature 0x13 (Tram Types)
+#
+
+properties[0x13] = {
+    **common_tracktype_props,
+    'powered_tramtype_list'    : {'custom_function': lambda x: label_list(x, 0x0F, "Tramtype")},
+    'tramtype_flags'           : {'size': 1, 'num': 0x10},
+    'speed_limit'              : {'size': 2, 'num': 0x14, 'unit_type': 'speed', 'unit_conversion': (10000, 1397)},
+    'requires_tramtype_list'   : {'custom_function': lambda x: label_list(x, 0x18, "Tramtype")},
+    'introduces_tramtype_list' : {'custom_function': lambda x: label_list(x, 0x19, "Tramtype")},
+    'alternative_tramtype_list': {'custom_function': lambda x: label_list(x, 0x1D, "Tramtype")},
+}
