@@ -81,7 +81,7 @@ class Action2Layout(action2.Action2):
         file.end_sprite()
 
 
-class Action2LayoutSpriteType(object):
+class Action2LayoutSpriteType:
     GROUND   = 0
     BUILDING = 1
     CHILD    = 2
@@ -93,7 +93,7 @@ layout_sprite_types = {
     'childsprite' : Action2LayoutSpriteType.CHILD,
 }
 
-class Action2LayoutSprite(object):
+class Action2LayoutSprite:
     def __init__(self, feature, type, pos = None, extra_dicts = []):
         self.feature = feature
         self.type = type
@@ -191,32 +191,19 @@ class Action2LayoutSprite(object):
         elif self.get_param('recolour_mode') != 0 and not self.is_set('palette'):
             raise generic.ScriptError("'palette' must be set when 'recolour_mode' is not set to RECOLOUR_NONE.")
 
-        # add the constant terms first
+        # Add the constant terms first
         sprite_num = self.get_param('recolour_mode') << 14
         if self.get_param('always_draw'):
             sprite_num |= 1 << 30
         if self.sprite_from_action1:
             sprite_num |= 1 << 31
 
-        add_sprite = False
-        sprite = self.get_param('sprite')
-        if isinstance(sprite, expression.ConstantNumeric):
-            sprite_num |= sprite.value
-        else:
-            add_sprite = True
-
-        add_palette = False
-        palette = self.get_param('palette')
-        if isinstance(palette, expression.ConstantNumeric):
-            sprite_num |= palette.value << 16
-        else:
-            add_palette = True
-
-        expr = expression.ConstantNumeric(sprite_num, sprite.pos)
-        if add_sprite:
-            expr = expression.BinOp(nmlop.ADD, sprite, expr, sprite.pos)
-        if add_palette:
-            expr = expression.BinOp(nmlop.ADD, palette, expr, sprite.pos)
+        # Prepare the return value
+        expr = expression.ConstantNumeric(sprite_num, self.pos)
+        # Add the sprite
+        expr = expression.BinOp(nmlop.ADD, self.get_param('sprite'), expr, self.pos)
+        # Add the palette
+        expr = expression.BinOp(nmlop.ADD, expression.BinOp(nmlop.SHIFT_LEFT, self.get_param('palette'), expression.ConstantNumeric(16, self.pos), self.pos), expr, self.pos)
         return expr.reduce()
 
     def get_param(self, name):
@@ -294,6 +281,8 @@ class Action2LayoutSprite(object):
             if isinstance(value, expression.ConstantNumeric):
                 generic.check_range(value.value, 0, (1 << 14) - 1, "sprite", value.pos)
                 return value
+            if value.supported_by_actionD(raise_error=False):
+                return value
             self.create_register(name, value)
             return expression.ConstantNumeric(0)
 
@@ -313,10 +302,14 @@ class Action2LayoutSprite(object):
                 self.create_register(name, offset)
             return val
         else:
+            self.palette_from_action1 = False
             if isinstance(value, expression.ConstantNumeric):
                 generic.check_range(value.value, 0, (1 << 14) - 1, "palette", value.pos)
-            self.palette_from_action1 = False
-            return value
+                return value
+            if value.supported_by_actionD(raise_error=False):
+                return value
+            self.create_register(name, value)
+            return expression.ConstantNumeric(0)
 
     def _validate_always_draw(self, name, value):
         if not isinstance(value, expression.ConstantNumeric):
@@ -433,15 +426,17 @@ def get_layout_action2s(spritelayout, feature, spr_pos):
     action6.free_parameters.save()
     act6 = action6.Action6()
 
+    advanced = any(x.is_advanced_sprite() for x in building_sprites + [ground_sprite])
+
     offset = 4
     sprite_num = ground_sprite.get_sprite_number()
     sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
-    offset += ground_sprite.get_registers_size()
+    if advanced: offset += ground_sprite.get_registers_size()
 
     for sprite in building_sprites:
         sprite_num = sprite.get_sprite_number()
         sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
-        offset += sprite.get_registers_size()
+        if advanced: offset += sprite.get_registers_size()
         offset += 3 if sprite.type == Action2LayoutSpriteType.CHILD else 6
 
     if len(act6.modifications) > 0:
