@@ -15,7 +15,7 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 
 from nml.actions import action2, action2real, action2var_variables, action4, action6, actionD
 from nml import expression, generic, global_constants, nmlop
-from nml.ast import general
+from nml.ast import general, switch
 
 class Action2Var(action2.Action2):
     """
@@ -103,7 +103,7 @@ class Action2Var(action2.Action2):
         file.comment(self.default_comment)
         file.end_sprite()
 
-class VarAction2Var(object):
+class VarAction2Var:
     """
     Represents a variable for use in a (advanced) variational action2.
 
@@ -176,9 +176,14 @@ class VarAction2Var(object):
 # Class for var 7E procedure calls
 class VarAction2ProcCallVar(VarAction2Var):
     def __init__(self, sg_ref):
+        if not isinstance(action2.resolve_spritegroup(sg_ref.name), (switch.Switch, switch.RandomSwitch)):
+            raise generic.ScriptError("Block with name '{}' is not a valid procedure".format(sg_ref.name), sg_ref.pos)
+        if not sg_ref.is_procedure:
+            raise generic.ScriptError("Unexpected identifier encountered: '{}'".format(sg_ref.name), sg_ref.pos)
         VarAction2Var.__init__(self, 0x7E, 0, 0)
         # Reference to the called action2
         self.sg_ref = sg_ref
+        self.comment = str(sg_ref)
 
     def resolve_parameter(self, feature):
         self.parameter = self.sg_ref.get_action2_id(feature)
@@ -237,7 +242,7 @@ class VarAction2LoadTempVar(VarAction2Var, expression.Expression):
 
 # Temporary load and store classes used for spritelayout parameters
 # Register is allocated in a separate entity
-class VarAction2LayoutParam(object):
+class VarAction2LayoutParam:
     def __init__(self):
         self.register = None
         self.store_vars = []
@@ -285,20 +290,20 @@ class VarAction2StoreLayoutParam(VarAction2Var):
     def get_size(self):
         return 6
 
-class VarAction2Range(object):
+class VarAction2Range:
     def __init__(self, min, max, result, comment):
         self.min = min
         self.max = max
         self.result = result
         self.comment = comment
 
-class Modification(object):
+class Modification:
     def __init__(self, param, size, offset):
         self.param = param
         self.size = size
         self.offset = offset
 
-class Varaction2Parser(object):
+class Varaction2Parser:
     def __init__(self, feature):
         self.feature = feature # Depends on feature and var_range
         self.extra_actions = []
@@ -401,9 +406,6 @@ class Varaction2Parser(object):
 
     def preprocess_storageop(self, expr):
         assert isinstance(expr, expression.StorageOp)
-        max = 0xF if expr.info['perm'] else 0x10F
-        if isinstance(expr.register, expression.ConstantNumeric) and expr.register.value > max:
-            raise generic.ScriptError("Register number must be in range 0..{:d}, encountered {:d}.".format(max, expr.register.value), expr.pos)
         if expr.info['perm'] and self.feature not in (0x08, 0x0A, 0x0D):
             raise generic.ScriptError("Persistent storage is not supported for feature '{}'".format(general.feature_name(self.feature)), expr.pos)
 
@@ -619,6 +621,9 @@ class Varaction2Parser(object):
             self.var_list.append(expr)
             self.var_list_size += expr.get_size()
 
+        elif isinstance(expr, expression.SpriteGroupRef):
+            self.parse_proc_call(expr)
+
         else:
             expr.supported_by_action2(True)
             assert False #supported_by_action2 should have raised the correct error already
@@ -972,6 +977,8 @@ def parse_varaction2(switch_block):
         act6.modify_bytes(mod.param, mod.size, mod.offset + offset)
     varaction2.var_list = parser.var_list
     offset += parser.var_list_size + 1 # +1 for the byte num-ranges
+    for proc in parser.proc_call_list:
+        action2.add_ref(proc, varaction2, True)
 
     none_result = None
     if any(x is not None and x.value is None for x in [r.result for r in switch_block.body.ranges] + [switch_block.body.default]):
