@@ -52,10 +52,10 @@ class ActionD(base_action.BaseAction):
         if self.data is not None: size += 4
 
         #print the statement for easier debugging
-        str1 = "param[%s]" % self.param1 if self.param1.value != 0xFF else str(self.data)
-        str2 = "param[%s]" % self.param2 if self.param2.value != 0xFF else str(self.data)
+        str1 = "param[{}]".format(self.param1) if self.param1.value != 0xFF else str(self.data)
+        str2 = "param[{}]".format(self.param2) if self.param2.value != 0xFF else str(self.data)
         str_total = self.op.to_string(str1, str2) if self.op != nmlop.ASSIGN else str1
-        file.comment("param[%s] = %s" % (self.target, str_total))
+        file.comment("param[{}] = {}".format(self.target, str_total))
 
         file.start_sprite(size)
         file.print_bytex(0x0D)
@@ -92,15 +92,15 @@ class ParameterAssignment(base_statement.BaseStatement):
         self.param = self.param.reduce(global_constants.const_list, unknown_id_fatal = False)
         if isinstance(self.param, expression.SpecialParameter):
             if not self.param.can_assign():
-                raise generic.ScriptError("Trying to assign a value to the read-only variable '%s'" % self.param.name, self.param.pos)
+                raise generic.ScriptError("Trying to assign a value to the read-only variable '{}'".format(self.param.name), self.param.pos)
         elif isinstance(self.param, expression.Identifier):
-            num = action6.free_parameters.pop_unique()
+            num = action6.free_parameters.pop_unique(self.pos)
             global_constants.named_parameters[self.param.value] = num
         elif not isinstance(self.param, expression.Parameter):
             raise generic.ScriptError("Left side of an assignment must be a parameter.", self.param.pos)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Parameter assignment'
+        generic.print_dbg(indentation, 'Parameter assignment')
         self.param.debug_print(indentation + 2)
         self.value.debug_print(indentation + 2)
 
@@ -108,7 +108,7 @@ class ParameterAssignment(base_statement.BaseStatement):
         return parse_actionD(self)
 
     def __str__(self):
-        return '%s = %s;\n' % (str(self.param), str(self.value))
+        return '{} = {};\n'.format(self.param, self.value)
 
 #prevent evaluating common sub-expressions multiple times
 def parse_subexpression(expr, action_list):
@@ -122,10 +122,47 @@ def parse_subexpression(expr, action_list):
 
 #returns a (param_num, action_list) tuple.
 def get_tmp_parameter(expr):
-    param = action6.free_parameters.pop()
+    param = action6.free_parameters.pop(expr.pos)
     actions = parse_actionD(ParameterAssignment(expression.Parameter(expression.ConstantNumeric(param)), expr))
     return (param, actions)
 
+def write_action_value(expr, action_list, act6, offset, size):
+    """
+    Write a single numeric value that is part of an action
+    Possibly use action6 and/or actionD if needed
+
+    @param expr: Expression to write
+    @type expr: L{Expression}
+
+    @param action_list: Action list to append any extra actions to
+    @type action_list: C{list} of L{BaseAction}
+
+    @param act6: Action6 to append any modifications to
+    @type act6: L{Action6}
+
+    @param offset: Offset to use for the action 6 (if applicable)
+    @type offset: C{int}
+
+    @param size: Size to use for the action 6 (if applicable)
+    @type size: C{int}
+
+    @return: 2-tuple containing -
+        - the value to be used when writing the action (0 if overwritten by action 6)
+        - the offset just past this numeric value (=offset + size)
+    @rtype: C{tuple} of (L{ConstantNumeric}, C{int})
+    """
+    if isinstance(expr, expression.ConstantNumeric):
+        result = expr
+    elif isinstance(expr, expression.Parameter) and isinstance(expr.num, expression.ConstantNumeric):
+        act6.modify_bytes(expr.num.value, size, offset)
+        result = expression.ConstantNumeric(0)
+    else:
+        tmp_param, tmp_param_actions = get_tmp_parameter(expr)
+        action_list.extend(tmp_param_actions)
+        act6.modify_bytes(tmp_param, size, offset)
+        result = expression.ConstantNumeric(0)
+    return result, offset + size
+
 def parse_ternary_op(assignment):
     assert isinstance(assignment.value, expression.TernaryOp)
     actions = parse_actionD(ParameterAssignment(assignment.param, assignment.value.expr2))
@@ -139,12 +176,13 @@ def parse_special_check(assignment):
     actions = parse_actionD(ParameterAssignment(assignment.param, expression.ConstantNumeric(check.results[0])))
 
     value = check.value
-    size = 4
     if check.mask is not None:
         value &= check.mask
         value += check.mask << 32
-        size = 8
-    actions.append(nml.actions.action7.SkipAction(9, check.varnum, size, check.op, value, 1))
+        assert check.varsize == 8
+    else:
+        assert check.varsize <= 4
+    actions.append(nml.actions.action7.SkipAction(9, check.varnum, check.varsize, check.op, value, 1))
 
     actions.extend(parse_actionD(ParameterAssignment(assignment.param, expression.ConstantNumeric(check.results[1]))))
     return actions
