@@ -15,7 +15,7 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 
 from nml import generic
 
-class Type(object):
+class Type:
     """
     Enum-type class of the various value types possible in NML
     """
@@ -25,7 +25,7 @@ class Type(object):
     FUNCTION_PTR = 3
     SPRITEGROUP_REF = 4
 
-class Expression(object):
+class Expression:
     """
     Superclass for all expression classes.
 
@@ -101,6 +101,15 @@ class Expression(object):
         if raise_error: raise generic.ScriptError("This expression can not be assigned to a parameter", self.pos)
         return False
 
+    def collect_references(self):
+        """
+        This function should collect all references to other nodes from this instance.
+
+        @return: A collection containing all links to other nodes.
+        @rtype: C{iterable} of L{SpriteGroupRef}
+        """
+        return []
+
     def is_boolean(self):
         """
         Check if this expression is limited to 0 or 1 as value.
@@ -121,6 +130,8 @@ class ConstantNumeric(Expression):
     def __init__(self, value, pos = None):
         Expression.__init__(self, pos)
         self.value = generic.truncate_int32(value)
+        self.uvalue = self.value
+        if self.uvalue < 0: self.uvalue += 2**32
 
     def debug_print(self, indentation):
         generic.print_dbg(indentation, 'Int:', self.value)
