@@ -39,17 +39,18 @@ class SpriteGroupRef(Expression):
         self.param_list = param_list
         self.pos = pos
         self.act2 = act2
+        self.is_procedure = False
 
     def debug_print(self, indentation):
-        print indentation*' ' +'Reference to: ' + str(self.name)
+        generic.print_dbg(indentation, 'Reference to:', self.name)
         if len(self.param_list) != 0:
-            print 'Parameters:'
+            generic.print_dbg(indentation, 'Parameters:')
             for p in self.param_list:
                 p.debug_print(indentation + 2)
 
     def __str__(self):
         if self.param_list:
-            return '%s(%s)' % (self.name, ', '.join(str(x) for x in self.param_list))
+            return '{}({})'.format(self.name, ', '.join(str(x) for x in self.param_list))
         return str(self.name)
 
     def get_action2_id(self, feature):
@@ -67,14 +68,22 @@ class SpriteGroupRef(Expression):
         try:
             spritegroup = action2.resolve_spritegroup(self.name)
         except generic.ScriptError:
-            assert False, "Illegal action2 reference '%s' encountered." % self.name.value
+            raise AssertionError("Illegal action2 reference '{}' encountered.".format(self.name.value))
 
         return spritegroup.get_action2(feature).id
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
         return self
 
+    def supported_by_action2(self, raise_error):
+        return True
+
+    def collect_references(self):
+        if self.is_procedure: return [self]
+        return []
+
     def type(self):
+        if self.is_procedure: return Type.INTEGER
         return Type.SPRITEGROUP_REF
 
     def __eq__(self, other):
