@@ -334,6 +334,7 @@ constant_numbers = {
 
     #industry tile special flags
     'INDTILE_FLAG_RANDOM_ANIMATION'   : 0,
+    'INDTILE_FLAG_ACCEPT_ALL'         : 1,
 
     'CB_RESULT_IND_PROD_NO_CHANGE'      : 0x00,
     'CB_RESULT_IND_PROD_HALF'           : 0x01,
@@ -390,6 +391,7 @@ constant_numbers = {
     'IND_FLAG_RANDOM_BITS_IN_PRODUCTION_CALLBACK'      : 15,
     'IND_FLAG_DO_NOT_FORCE_INSTANCE_AT_MAP_GENERATION' : 16,
     'IND_FLAG_ALLOW_CLOSING_LAST_INSTANCE'             : 17,
+    'IND_FLAG_LONG_CARGO_TYPE_LISTS'                   : 18,
 
     #flags for builtin function industry_type(..)
     'IND_TYPE_OLD'             : 0,
@@ -426,6 +428,24 @@ constant_numbers = {
     #railtype flags
     'RAILTYPE_FLAG_CATENARY'          : 0,
     'RAILTYPE_FLAG_NO_LEVEL_CROSSING' : 1, # for OpenTTD > r20049
+    'RAILTYPE_FLAG_HIDDEN'            : 2,
+    'RAILTYPE_FLAG_PRECOMBINED'       : 3,
+    'RAILTYPE_FLAG_ALLOW_90DEG'       : 4,
+    'RAILTYPE_FLAG_DISALLOW_90DEG'    : 5,
+
+    #roadtype flags
+    'ROADTYPE_FLAG_CATENARY'          : 0,
+    'ROADTYPE_FLAG_NO_LEVEL_CROSSING' : 1,
+    'ROADTYPE_FLAG_NO_HOUSES'         : 2,
+    'ROADTYPE_FLAG_HIDDEN'            : 3,
+    'ROADTYPE_FLAG_TOWN_BUILD'        : 4,
+
+    #tramtype flags
+    'TRAMTYPE_FLAG_CATENARY'          : 0,
+    'TRAMTYPE_FLAG_NO_LEVEL_CROSSING' : 1,
+    'TRAMTYPE_FLAG_NO_HOUSES'         : 2,
+    'TRAMTYPE_FLAG_HIDDEN'            : 3,
+    'TRAMTYPE_FLAG_TOWN_BUILD'        : 4,
 
     #type of default station graphics used for a railtype
     'RAILTYPE_STATION_NORMAL'   : 0,
@@ -698,6 +718,7 @@ constant_numbers = {
     'ANIM_TRIGGER_APT_NEW_CARGO'            : 2,
     'ANIM_TRIGGER_APT_CARGO_TAKEN'          : 3,
     'ANIM_TRIGGER_APT_250_TICKS'            : 4,
+    'ANIM_TRIGGER_APT_AIRPLANE_LANDS'       : 5,
 
     # Animation looping
     'ANIMATION_NON_LOOPING'                 : 0,
@@ -731,7 +752,7 @@ constant_numbers = {
     'PALETTE_CC_PALE_GREEN'                 : 776,
     'PALETTE_CC_PINK'                       : 777,
     'PALETTE_CC_YELLOW'                     : 778,
-    'PALETTE_CC_RED'                        : 778,
+    'PALETTE_CC_RED'                        : 779,
     'PALETTE_CC_LIGHT_BLUE'                 : 780,
     'PALETTE_CC_GREEN'                      : 781,
     'PALETTE_CC_DARK_GREEN'                 : 782,
@@ -1168,6 +1189,7 @@ config_flags = {
     'newcargos'               : 0x6B,
     'dynamic_engines'         : 0x78,
     'variable_runningcosts'   : 0x7E,
+    '256_persistent_registers': 0x80,
 }
 
 def unified_maglev_read(info, pos):
@@ -1198,8 +1220,19 @@ def create_spritegroup_ref(info, pos):
     return expression.SpriteGroupRef(expression.Identifier(info), [], pos)
 
 cargo_numbers = {}
+
 is_default_railtype_table = True
+# if no railtype_table is provided, OpenTTD assumes these 4 railtypes
 railtype_table = {'RAIL': 0, 'ELRL': 1, 'MONO': 1, 'MGLV': 2}
+
+is_default_roadtype_table = True
+# if no roadtype_table is provided, OpenTTD sets all vehicles to ROAD
+roadtype_table = {'ROAD': 0}
+
+is_default_tramtype_table = True
+# if no tramtype_table is provided, OpenTTD sets all vehicles to ELRL
+tramtype_table = {'ELRL': 0}
+
 item_names = {}
 settings = {}
 named_parameters = {}
@@ -1213,6 +1246,8 @@ const_list = [
     (named_parameters, param_from_name),
     cargo_numbers,
     railtype_table,
+    roadtype_table,
+    tramtype_table,
     (item_names, item_to_id),
     (settings, setting_from_info),
     (config_flags, config_flag),
@@ -1229,3 +1264,7 @@ def print_stats():
         generic.print_info("Cargo translation table: {}/{}".format(len(cargo_numbers), 0xFE))
     if not is_default_railtype_table:
         generic.print_info("Railtype translation table: {}/{}".format(len(railtype_table), 0x100))
+    if not is_default_roadtype_table:
+        generic.print_info("Roadtype translation table: {}/{}".format(len(roadtype_table), 0x100))
+    if not is_default_tramtype_table:
+        generic.print_info("Tramtype translation table: {}/{}".format(len(tramtype_table), 0x100))
