@@ -21,10 +21,7 @@ from nml.ast import grf, alt_sprites
 try:
     from PIL import Image
 except ImportError:
-    try:
-        import Image
-    except ImportError:
-        pass
+    pass
 
 developmode = False # Give 'nice' error message instead of a stack dump.
 
@@ -43,7 +40,8 @@ def parse_cli(argv):
     opt_parser = optparse.OptionParser(usage=usage, version=version_info.get_cli_version())
     opt_parser.set_defaults(debug=False, crop=False, compress=True, outputs=[], start_sprite_num=0,
                             custom_tags="custom_tags.txt", lang_dir="lang", default_lang="english.lng", cache_dir=".nmlcache",
-                            forced_palette="ANY", quiet=False, md5_filename=None, keep_orphaned=True, verbosity=generic.verbosity_level)
+                            forced_palette="ANY", quiet=False, md5_filename=None, keep_orphaned=True, verbosity=generic.verbosity_level,
+                            rebuild_parser=False, debug_parser=False)
     opt_parser.add_option("-d", "--debug", action="store_true", dest="debug", help="write the AST to stdout")
     opt_parser.add_option("-s", "--stack", action="store_true", dest="stack", help="Dump stack when an error occurs")
     opt_parser.add_option("--grf", dest="grf_filename", metavar="<file>", help="write the resulting grf to <file>")
@@ -73,11 +71,13 @@ def parse_cli(argv):
     opt_parser.add_option("--cache-dir", dest="cache_dir", metavar="<dir>", help="Cache files are stored in directory <dir> [default: %default]")
     opt_parser.add_option("--clear-orphaned", action="store_false", dest="keep_orphaned", help="Remove unused/orphaned items from cache files.")
     opt_parser.add_option("--verbosity", type="int", dest="verbosity", metavar="<level>", help="Set the verbosity level for informational output. [default: %default, max: {}]".format(generic.VERBOSITY_MAX))
+    opt_parser.add_option("-R", "--rebuild-parser", action="store_true", dest="rebuild_parser", help="Force regeneration of parser tables.")
+    opt_parser.add_option("-D", "--debug-parser", action="store_true", dest="debug_parser", help="Enable debug mode for parser.")
 
     opts, args = opt_parser.parse_args(argv)
 
     generic.set_verbosity(0 if opts.quiet else opts.verbosity)
-    generic.set_cache_root_dir(opts.cache_dir)
+    generic.set_cache_root_dir(None if opts.no_cache else opts.cache_dir)
     spritecache.keep_orphaned = opts.keep_orphaned
 
     opts.outputfile_given = (opts.grf_filename or opts.nfo_filename or opts.nml_filename or opts.dep_filename or opts.outputs)
@@ -167,7 +167,7 @@ def main(argv):
             generic.print_error("Unknown output format {}".format(outext))
             sys.exit(2)
 
-    ret = nml(input, input_filename, opts.debug, outputs, opts.start_sprite_num, opts.compress, opts.crop, not opts.no_cache, opts.forced_palette, opts.md5_filename)
+    ret = nml(input, input_filename, opts.debug, outputs, opts.start_sprite_num, opts.compress, opts.crop, opts.forced_palette, opts.md5_filename, opts.rebuild_parser, opts.debug_parser)
 
     input.close()
     sys.exit(ret)
@@ -175,7 +175,7 @@ def main(argv):
 def filename_output_from_input(name, ext):
     return os.path.splitext(name)[0] + ext
 
-def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num, compress_grf, crop_sprites, enable_cache, forced_palette, md5_filename):
+def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num, compress_grf, crop_sprites, forced_palette, md5_filename, rebuild_parser, debug_parser):
     """
     Compile an NML file.
 
@@ -197,9 +197,6 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
     @param crop_sprites: Enable sprite cropping.
     @type  crop_sprites: C{bool}
 
-    @param enable_cache: Enable sprite cache.
-    @type  enable_cache: C{bool}
-
     @param forced_palette: Palette to use for the file.
     @type  forced_palette: C{str}
 
@@ -223,7 +220,7 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
 
     generic.print_progress("Init parser ...")
 
-    nml_parser = parser.NMLParser()
+    nml_parser = parser.NMLParser(rebuild_parser, debug_parser)
     if input_filename is None:
         input_filename = 'input'
 
@@ -237,9 +234,8 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
 
     for outputfile in outputfiles:
         if isinstance(outputfile, output_nml.OutputNML):
-            outputfile.open()
-            outputfile.write(str(result))
-            outputfile.close()
+            with outputfile:
+                outputfile.write(str(result))
 
     generic.print_progress("Preprocessing ...")
 
@@ -304,13 +300,12 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
     skip_sprite_processing = True
     for outputfile in outputfiles:
         if isinstance(outputfile, output_dep.OutputDEP):
-            outputfile.open()
-            for f in sprite_files:
-                if f[0] is not None:
-                    outputfile.write(f[0])
-                if f[1] is not None:
-                    outputfile.write(f[1])
-            outputfile.close()
+            with outputfile:
+                for f in sprite_files:
+                    if f[0] is not None:
+                        outputfile.write(f[0])
+                    if f[1] is not None:
+                        outputfile.write(f[1])
         skip_sprite_processing &= outputfile.skip_sprite_checks()
 
     if skip_sprite_processing:
@@ -332,12 +327,13 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
             f = f_pair[0]
 
         try:
-            im = Image.open(generic.find_file(f))
+            with Image.open(generic.find_file(f)) as im:
+                # Verify the image is running in Palette mode, if not, skip this file.
+                if im.mode != "P":
+                    continue
+                pal = palette.validate_palette(im, f)
         except IOError as ex:
             raise generic.ImageError(str(ex), f)
-        if im.mode != "P":
-            continue
-        pal = palette.validate_palette(im, f)
 
         if forced_palette != "ANY" and pal != forced_palette and not (forced_palette == "DEFAULT" and pal == "LEGACY"):
             raise generic.ImageError("Image has '{}' palette, but you forced the '{}' palette".format(pal, used_palette), f)
@@ -359,7 +355,7 @@ def nml(inputfile, input_filename, output_debug, outputfiles, start_sprite_num,
         outputfile.palette = used_palette # used by RecolourSpriteAction
         if isinstance(outputfile, output_grf.OutputGRF):
             if encoder is None:
-                encoder = spriteencoder.SpriteEncoder(compress_grf, crop_sprites, enable_cache, used_palette)
+                encoder = spriteencoder.SpriteEncoder(compress_grf, crop_sprites, used_palette)
             outputfile.encoder = encoder
 
     generic.clear_progress()
