@@ -18,7 +18,7 @@ Abstract base classes that implements common functionality for output classes
 """
 import array, io
 
-class OutputBase(object):
+class OutputBase:
     """
     Base class for output to a data file.
 
@@ -88,6 +88,19 @@ class OutputBase(object):
         """
         return False
 
+    def __enter__(self):
+        """
+        Allow `OutputBase` and subclasses to be used as context managers.
+        """
+        self.open()
+        return self
+
+    def __exit__(self, type, value, traceback):
+        """
+        Allow `OutputBase` and subclasses to be used as context managers.
+        """
+        self.close()
+
 
 class SpriteOutputBase(OutputBase):
     """
